@@ -5,15 +5,14 @@
 # License: BSD-3-Clause
 
 from copy import deepcopy
-from distutils.version import LooseVersion
 import itertools as itt
 from math import log
-import os
 
 import numpy as np
 
-from .defaults import _EXTRAPOLATE_DEFAULT, _BORDER_DEFAULT, DEFAULTS
-from .io.write import start_file, end_file
+from .defaults import (_INTERPOLATION_DEFAULT, _EXTRAPOLATE_DEFAULT,
+                       _BORDER_DEFAULT, DEFAULTS)
+from .io.write import start_and_end_file
 from .io.proj import (make_projector, _proj_equal, activate_proj,
                       _check_projs, _needs_eeg_average_ref_proj,
                       _has_eeg_average_ref_proj, _read_proj, _write_proj)
@@ -38,8 +37,9 @@ from .utils import (check_fname, logger, verbose, check_version, _time_mask,
                     warn, copy_function_doc_to_method_doc, _pl,
                     _undo_scaling_cov, _scaled_array, _validate_type,
                     _check_option, eigh, fill_doc, _on_missing,
-                    _check_on_missing, _check_fname)
+                    _check_on_missing, _check_fname, _verbose_safe_false)
 from . import viz
+from .viz.utils import _warn_deprecated_vmin_vmax
 
 from .fixes import (BaseEstimator, EmpiricalCovariance, _logdet,
                     empirical_covariance, log_likelihood)
@@ -69,9 +69,10 @@ def _get_tslice(epochs, tmin, tmax):
 class Covariance(dict):
     """Noise covariance matrix.
 
-    .. warning:: This class should not be instantiated directly, but
-                 instead should be created using a covariance reading or
-                 computation function.
+    .. note::
+        This class should not be instantiated directly via
+        ``mne.Covariance(...)``. Instead, use one of the functions
+        listed in the See Also section below.
 
     Parameters
     ----------
@@ -93,7 +94,7 @@ class Covariance(dict):
         The method used to compute the covariance.
     loglik : float
         The log likelihood.
-    %(verbose_meth)s
+    %(verbose)s
 
     Attributes
     ----------
@@ -114,8 +115,9 @@ class Covariance(dict):
     read_cov
     """
 
+    @verbose
     def __init__(self, data, names, bads, projs, nfree, eig=None, eigvec=None,
-                 method=None, loglik=None, verbose=None):
+                 method=None, loglik=None, *, verbose=None):
         """Init of covariance."""
         diag = (data.ndim == 1)
         projs = _check_projs(projs)
@@ -126,7 +128,6 @@ class Covariance(dict):
             self['method'] = method
         if loglik is not None:
             self['loglik'] = loglik
-        self.verbose = verbose
 
     @property
     def data(self):
@@ -143,28 +144,24 @@ class Covariance(dict):
         """Number of degrees of freedom."""
         return self['nfree']
 
-    def save(self, fname):
+    @verbose
+    def save(self, fname, *, overwrite=False, verbose=None):
         """Save covariance matrix in a FIF file.
 
         Parameters
         ----------
         fname : str
             Output filename.
+        %(overwrite)s
+
+            .. versionadded:: 1.0
+        %(verbose)s
         """
         check_fname(fname, 'covariance', ('-cov.fif', '-cov.fif.gz',
                                           '_cov.fif', '_cov.fif.gz'))
-        # TODO: Add `overwrite` param to method signature
-        fname = _check_fname(fname=fname, overwrite=True)
-        fid = start_file(fname)
-
-        try:
+        fname = _check_fname(fname=fname, overwrite=overwrite)
+        with start_and_end_file(fname) as fid:
             _write_cov(fid, self)
-        except Exception:
-            fid.close()
-            os.remove(fname)
-            raise
-
-        end_file(fid)
 
     def copy(self):
         """Copy the Covariance object.
@@ -257,47 +254,63 @@ class Covariance(dict):
                                  show, verbose)
 
     @verbose
-    def plot_topomap(self, info, ch_type=None, vmin=None,
-                     vmax=None, cmap=None, sensors=True, colorbar=True,
-                     scalings=None, units=None, res=64,
-                     size=1, cbar_fmt="%3.1f",
-                     proj=False, show=True, show_names=False, title=None,
-                     mask=None, mask_params=None, outlines='head',
-                     contours=6, image_interp='bilinear',
-                     axes=None, extrapolate=_EXTRAPOLATE_DEFAULT, sphere=None,
-                     border=_BORDER_DEFAULT,
-                     noise_cov=None, verbose=None):
+    def plot_topomap(
+            self, info, ch_type=None, *, scalings=None, proj=False,
+            noise_cov=None, sensors=True, show_names=False, mask=None,
+            mask_params=None, contours=6, outlines='head', sphere=None,
+            image_interp=_INTERPOLATION_DEFAULT,
+            extrapolate=_EXTRAPOLATE_DEFAULT, border=_BORDER_DEFAULT, res=64,
+            size=1, cmap=None, vlim=(None, None), vmin=None, vmax=None,
+            cnorm=None, colorbar=True, cbar_fmt='%3.1f', units=None,
+            axes=None, title=None, show=True, verbose=None):
         """Plot a topomap of the covariance diagonal.
 
         Parameters
         ----------
         %(info_not_none)s
-        %(topomap_ch_type)s
-        %(topomap_vmin_vmax)s
-        %(topomap_cmap)s
-        %(topomap_sensors)s
-        %(topomap_colorbar)s
-        %(topomap_scalings)s
-        %(topomap_units)s
-        %(topomap_res)s
-        %(topomap_size)s
-        %(topomap_cbar_fmt)s
-        %(plot_proj)s
-        %(show)s
-        %(topomap_show_names)s
-        %(title_None)s
-        %(topomap_mask)s
-        %(topomap_mask_params)s
-        %(topomap_outlines)s
-        %(topomap_contours)s
-        %(topomap_image_interp)s
-        %(topomap_axes)s
-        %(topomap_extrapolate)s
-        %(topomap_sphere_auto)s
-        %(topomap_border)s
+        %(ch_type_topomap)s
+
+            .. versionadded:: 0.21
+        %(scalings_topomap)s
+        %(proj_plot)s
         noise_cov : instance of Covariance | None
             If not None, whiten the instance with ``noise_cov`` before
             plotting.
+        %(sensors_topomap)s
+        %(show_names_topomap)s
+        %(mask_topomap)s
+        %(mask_params_topomap)s
+        %(contours_topomap)s
+        %(outlines_topomap)s
+        %(sphere_topomap_auto)s
+        %(image_interp_topomap)s
+        %(extrapolate_topomap)s
+        %(border_topomap)s
+        %(res_topomap)s
+        %(size_topomap)s
+        %(cmap_topomap)s
+        %(vlim_plot_topomap)s
+
+            .. versionadded:: 1.2
+        %(vmin_vmax_topomap)s
+
+            .. deprecated:: v1.2
+               The ``vmin`` and ``vmax`` parameters will be removed in version
+               1.3. Please use the ``vlim`` parameter instead.
+        %(cnorm)s
+
+            .. versionadded:: 1.2
+        %(colorbar_topomap)s
+        %(cbar_fmt_topomap)s
+        %(units_topomap_evoked)s
+        %(axes_cov_plot_topomap)s
+        %(title_none)s
+
+            .. deprecated:: v1.2
+               The ``title`` parameter will be removed in version 1.3. Please
+               use :meth:`fig.suptitle()<matplotlib.figure.Figure.suptitle>`
+               instead.
+        %(show)s
         %(verbose)s
 
         Returns
@@ -310,6 +323,9 @@ class Covariance(dict):
         .. versionadded:: 0.21
         """
         from .viz.misc import _index_info_cov
+
+        vlim = _warn_deprecated_vmin_vmax(vlim, vmin, vmax)
+
         info, C, _, _ = _index_info_cov(info, self, exclude=())
         evoked = EvokedArray(np.diag(C)[:, np.newaxis], info)
         if noise_cov is not None:
@@ -325,8 +341,8 @@ class Covariance(dict):
         if scalings is None:
             scalings = {k: v * v for k, v in DEFAULTS['scalings'].items()}
         return evoked.plot_topomap(
-            times=[0], ch_type=ch_type, vmin=vmin, vmax=vmax, cmap=cmap,
-            sensors=sensors, colorbar=colorbar, scalings=scalings,
+            times=[0], ch_type=ch_type, vmin=vlim[0], vmax=vlim[1], cmap=cmap,
+            sensors=sensors, cnorm=cnorm, colorbar=colorbar, scalings=scalings,
             units=units, res=res, size=size, cbar_fmt=cbar_fmt,
             proj=proj, show=show, show_names=show_names, title=title,
             mask=mask, mask_params=mask_params, outlines=outlines,
@@ -445,9 +461,9 @@ def _check_n_samples(n_samples, n_chan):
 @verbose
 def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
                            flat=None, picks=None, method='empirical',
-                           method_params=None, cv=3, scalings=None, n_jobs=1,
-                           return_estimators=False, reject_by_annotation=True,
-                           rank=None, verbose=None):
+                           method_params=None, cv=3, scalings=None,
+                           n_jobs=None, return_estimators=False,
+                           reject_by_annotation=True, rank=None, verbose=None):
     """Estimate noise covariance matrix from a continuous segment of raw data.
 
     It is typically useful to estimate a noise covariance from empty room
@@ -520,7 +536,7 @@ def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
     %(reject_by_annotation_epochs)s
 
         .. versionadded:: 0.14
-    %(rank_None)s
+    %(rank_none)s
 
         .. versionadded:: 0.17
 
@@ -573,7 +589,8 @@ def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
         pick_mask = slice(None)
         picks = _picks_to_idx(raw.info, picks)
     epochs = Epochs(raw, events, 1, 0, tstep_m1, baseline=None,
-                    picks=picks, reject=reject, flat=flat, verbose=False,
+                    picks=picks, reject=reject, flat=flat,
+                    verbose=_verbose_safe_false(),
                     preload=False, proj=False,
                     reject_by_annotation=reject_by_annotation)
     if method is None:
@@ -684,8 +701,9 @@ def _check_method_params(method, method_params, keep_sample_mean=True,
 @verbose
 def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
                        projs=None, method='empirical', method_params=None,
-                       cv=3, scalings=None, n_jobs=1, return_estimators=False,
-                       on_mismatch='raise', rank=None, verbose=None):
+                       cv=3, scalings=None, n_jobs=None,
+                       return_estimators=False, on_mismatch='raise',
+                       rank=None, verbose=None):
     """Estimate noise covariance matrix from epochs.
 
     The noise covariance is typically estimated on pre-stimulus periods
@@ -772,7 +790,7 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
         unstable results in covariance calculation, e.g. when data
         have been processed with Maxwell filtering but not transformed
         to the same head position.
-    %(rank_None)s
+    %(rank_none)s
 
         .. versionadded:: 0.17
 
@@ -1017,21 +1035,14 @@ def _eigvec_subspace(eig, eigvec, mask):
     return eig, eigvec
 
 
-def _get_iid_kwargs():
-    import sklearn
-    kwargs = dict()
-    if LooseVersion(sklearn.__version__) < LooseVersion('0.22'):
-        kwargs['iid'] = False
-    return kwargs
-
-
 def _compute_covariance_auto(data, method, info, method_params, cv,
                              scalings, n_jobs, stop_early, picks_list, rank):
     """Compute covariance auto mode."""
     # rescale to improve numerical stability
     orig_rank = rank
-    rank = compute_rank(RawArray(data.T, info, copy=None, verbose=False),
-                        rank, scalings, info)
+    rank = compute_rank(
+        RawArray(data.T, info, copy=None, verbose=_verbose_safe_false()),
+        rank, scalings, info)
     with _scaled_array(data.T, picks_list, scalings):
         C = np.dot(data.T, data)
         _, eigvec, mask = _smart_eigh(C, info, rank, proj_subspace=True,
@@ -1110,7 +1121,7 @@ def _compute_covariance_auto(data, method, info, method_params, cv,
                 tuned_parameters = [{'shrinkage': shrinkage}]
                 shrinkages = []
                 gs = GridSearchCV(ShrunkCovariance(**mp),
-                                  tuned_parameters, cv=cv, **_get_iid_kwargs())
+                                  tuned_parameters, cv=cv)
                 for ch_type, picks in sub_picks_list:
                     gs.fit(data_[:, picks])
                     shrinkages.append((ch_type, gs.best_estimator_.shrinkage,
@@ -1376,7 +1387,8 @@ class _ShrunkCovariance(BaseEstimator):
 ###############################################################################
 # Writing
 
-def write_cov(fname, cov):
+@verbose
+def write_cov(fname, cov, *, overwrite=False, verbose=None):
     """Write a noise covariance matrix.
 
     Parameters
@@ -1385,12 +1397,16 @@ def write_cov(fname, cov):
         The name of the file. It should end with -cov.fif or -cov.fif.gz.
     cov : Covariance
         The noise covariance matrix.
+    %(overwrite)s
+
+        .. versionadded:: 1.0
+    %(verbose)s
 
     See Also
     --------
     read_cov
     """
-    cov.save(fname)
+    cov.save(fname, overwrite=overwrite, verbose=verbose)
 
 
 ###############################################################################
@@ -1437,7 +1453,7 @@ def prepare_noise_cov(noise_cov, info, ch_names=None, rank=None,
     ch_names : list | None
         The channel names to be considered. Can be None to use
         ``info['ch_names']``.
-    %(rank_None)s
+    %(rank_none)s
 
         .. versionadded:: 0.18
            Support for 'info' mode.
@@ -1538,7 +1554,7 @@ def _smart_eigh(C, info, rank, scalings=None, projs=None,
         eig[picks], eigvec[np.ix_(picks, picks)], mask[picks] = e, ev, m
         # XXX : also handle ref for sEEG and ECoG
         if ch_type == 'eeg' and _needs_eeg_average_ref_proj(info) and not \
-                _has_eeg_average_ref_proj(projs):
+                _has_eeg_average_ref_proj(info, projs=projs):
             warn('No average EEG reference present in info["projs"], '
                  'covariance may be adversely affected. Consider recomputing '
                  'covariance using with an average eeg reference projector '
@@ -1602,7 +1618,7 @@ def regularize(cov, info, mag=0.1, grad=0.1, eeg=0.1, exclude='bads',
         Regularization factor for EEG-CSD signals.
     dbs : float (default 0.1)
         Regularization factor for DBS signals.
-    %(rank_None)s
+    %(rank_none)s
 
         .. versionadded:: 0.17
 
@@ -1757,7 +1773,7 @@ def _regularized_covariance(data, reg=None, method_params=None, info=None,
     scalings = _handle_default('scalings_cov_rank', None)
     cov = _compute_covariance_auto(
         data.T, method=method, method_params=method_params,
-        info=info, cv=None, n_jobs=1, stop_early=True,
+        info=info, cv=None, n_jobs=None, stop_early=True,
         picks_list=picks_list, scalings=scalings,
         rank=rank)[reg]['data']
     return cov
@@ -1777,7 +1793,7 @@ def compute_whitener(noise_cov, info=None, picks=None, rank=None,
     %(info)s Can be None if ``noise_cov`` has already been
         prepared with :func:`prepare_noise_cov`.
     %(picks_good_data_noref)s
-    %(rank_None)s
+    %(rank_none)s
 
         .. versionadded:: 0.18
            Support for 'info' mode.
@@ -1889,7 +1905,7 @@ def whiten_evoked(evoked, noise_cov, picks=None, diag=None, rank=None,
     %(picks_good_data)s
     diag : bool (default False)
         If True, whiten using only the diagonal of the covariance.
-    %(rank_None)s
+    %(rank_none)s
 
         .. versionadded:: 0.18
            Support for 'info' mode.
@@ -2085,3 +2101,12 @@ def _write_cov(fid, cov):
 
     #   Done!
     end_block(fid, FIFF.FIFFB_MNE_COV)
+
+
+@verbose
+def _ensure_cov(cov, name='cov', *, verbose=None):
+    _validate_type(cov, ('path-like', Covariance), name)
+    logger.info('Noise covariance  : %s' % (cov,))
+    if not isinstance(cov, Covariance):
+        cov = read_cov(cov, verbose=_verbose_safe_false())
+    return cov
