@@ -7,12 +7,10 @@ from functools import partial
 import numpy as np
 
 from .annotations import _annotations_starts_stops
-from .fixes import _import_fft
 from .io.pick import _picks_to_idx
 from .cuda import (_setup_cuda_fft_multiply_repeated, _fft_multiply_repeated,
                    _setup_cuda_fft_resample, _fft_resample, _smart_pad)
-from .parallel import parallel_func, check_n_jobs
-from .time_frequency.multitaper import _mt_spectra, _compute_mt_params
+from .parallel import parallel_func
 from .utils import (logger, verbose, sum_squared, warn, _pl,
                     _check_preload, _validate_type, _check_option, _ensure_int)
 from ._ola import _COLA
@@ -124,7 +122,7 @@ def next_fast_len(target):
 
 
 def _overlap_add_filter(x, h, n_fft=None, phase='zero', picks=None,
-                        n_jobs=1, copy=True, pad='reflect_limited'):
+                        n_jobs=None, copy=True, pad='reflect_limited'):
     """Filter the signal x using h with overlap-add FFTs.
 
     Parameters
@@ -158,7 +156,6 @@ def _overlap_add_filter(x, h, n_fft=None, phase='zero', picks=None,
     x : array, shape (n_signals, n_times)
         x filtered.
     """
-    n_jobs = check_n_jobs(n_jobs, allow_cuda=True)
     # set up array for filtering, reshape to 2D, operate on last axis
     x, orig_shape, picks = _prep_for_filtering(x, copy, picks)
     # Extend the signal by mirroring the edges to reduce transient filter
@@ -198,17 +195,16 @@ def _overlap_add_filter(x, h, n_fft=None, phase='zero', picks=None,
                          '2 * len(h) - 1 (%s), got %s' % (min_fft, n_fft))
 
     # Figure out if we should use CUDA
-    n_jobs, cuda_dict = _setup_cuda_fft_multiply_repeated(
-        n_jobs, h, n_fft)
+    n_jobs, cuda_dict = _setup_cuda_fft_multiply_repeated(n_jobs, h, n_fft)
 
     # Process each row separately
     picks = _picks_to_idx(len(x), picks)
+    parallel, p_fun, _ = parallel_func(_1d_overlap_filter, n_jobs)
     if n_jobs == 1:
         for p in picks:
             x[p] = _1d_overlap_filter(x[p], len(h), n_edge, phase,
                                       cuda_dict, pad, n_fft)
     else:
-        parallel, p_fun, _ = parallel_func(_1d_overlap_filter, n_jobs)
         data_new = parallel(p_fun(x[p], len(h), n_edge, phase,
                                   cuda_dict, pad, n_fft) for p in picks)
         for pp, p in enumerate(picks):
@@ -311,7 +307,7 @@ def _firwin_design(N, freq, gain, window, sfreq):
                                  % (N, transition * sfreq / 2., this_N))
             # Construct a lowpass
             this_h = firwin(this_N, (prev_freq + this_freq) / 2.,
-                            window=window, pass_zero=True, nyq=freq[-1])
+                            window=window, pass_zero=True, fs=freq[-1] * 2)
             assert this_h.shape == (this_N,)
             offset = (N - this_N) // 2
             if this_gain == 0:
@@ -429,7 +425,6 @@ def _filtfilt(x, iir_params, picks, n_jobs, copy):
     # set up array for filtering, reshape to 2D, operate on last axis
     from scipy.signal import filtfilt, sosfiltfilt
     padlen = min(iir_params['padlen'], x.shape[-1] - 1)
-    n_jobs = check_n_jobs(n_jobs)
     x, orig_shape, picks = _prep_for_filtering(x, copy, picks)
     if 'sos' in iir_params:
         fun = partial(sosfiltfilt, sos=iir_params['sos'], padlen=padlen,
@@ -439,11 +434,11 @@ def _filtfilt(x, iir_params, picks, n_jobs, copy):
         fun = partial(filtfilt, b=iir_params['b'], a=iir_params['a'],
                       padlen=padlen, axis=-1)
         _check_coefficients((iir_params['b'], iir_params['a']))
+    parallel, p_fun, n_jobs = parallel_func(fun, n_jobs)
     if n_jobs == 1:
         for p in picks:
             x[p] = fun(x=x[p])
     else:
-        parallel, p_fun, _ = parallel_func(fun, n_jobs)
         data_new = parallel(p_fun(x=x[p]) for p in picks)
         for pp, p in enumerate(picks):
             x[p] = data_new[pp]
@@ -743,10 +738,10 @@ def _check_method(method, iir_params, extra_types=()):
 
 @verbose
 def filter_data(data, sfreq, l_freq, h_freq, picks=None, filter_length='auto',
-                l_trans_bandwidth='auto', h_trans_bandwidth='auto', n_jobs=1,
-                method='fir', iir_params=None, copy=True, phase='zero',
-                fir_window='hamming', fir_design='firwin',
-                pad='reflect_limited', verbose=None):
+                l_trans_bandwidth='auto', h_trans_bandwidth='auto',
+                n_jobs=None, method='fir', iir_params=None, copy=True,
+                phase='zero', fir_window='hamming', fir_design='firwin',
+                pad='reflect_limited', *, verbose=None):
     """Filter a subset of channels.
 
     Parameters
@@ -763,8 +758,8 @@ def filter_data(data, sfreq, l_freq, h_freq, picks=None, filter_length='auto',
     %(filter_length)s
     %(l_trans_bandwidth)s
     %(h_trans_bandwidth)s
-    %(n_jobs-fir)s
-    %(method-fir)s
+    %(n_jobs_fir)s
+    %(method_fir)s
     %(iir_params)s
     copy : bool
         If True, a copy of x, filtered, is returned. Otherwise, it operates
@@ -772,7 +767,7 @@ def filter_data(data, sfreq, l_freq, h_freq, picks=None, filter_length='auto',
     %(phase)s
     %(fir_window)s
     %(fir_design)s
-    %(pad-fir)s
+    %(pad_fir)s
         The default is ``'reflect_limited'``.
 
         .. versionadded:: 0.15
@@ -806,7 +801,7 @@ def filter_data(data, sfreq, l_freq, h_freq, picks=None, filter_length='auto',
 
     .. note:: If n_jobs > 1, more memory is required as
               ``len(picks) * n_times`` additional time points need to
-              be temporaily stored in memory.
+              be temporarily stored in memory.
 
     For more information, see the tutorials
     :ref:`disc-filtering` and :ref:`tut-filter-resample` and
@@ -853,7 +848,7 @@ def create_filter(data, sfreq, l_freq, h_freq, filter_length='auto',
     %(filter_length)s
     %(l_trans_bandwidth)s
     %(h_trans_bandwidth)s
-    %(method-fir)s
+    %(method_fir)s
     %(iir_params)s
     %(phase)s
     %(fir_window)s
@@ -1072,9 +1067,9 @@ def create_filter(data, sfreq, l_freq, h_freq, filter_length='auto',
 @verbose
 def notch_filter(x, Fs, freqs, filter_length='auto', notch_widths=None,
                  trans_bandwidth=1, method='fir', iir_params=None,
-                 mt_bandwidth=None, p_value=0.05, picks=None, n_jobs=1,
+                 mt_bandwidth=None, p_value=0.05, picks=None, n_jobs=None,
                  copy=True, phase='zero', fir_window='hamming',
-                 fir_design='firwin', pad='reflect_limited',
+                 fir_design='firwin', pad='reflect_limited', *,
                  verbose=None):
     r"""Notch filter for the signal x.
 
@@ -1098,7 +1093,7 @@ def notch_filter(x, Fs, freqs, filter_length='auto', notch_widths=None,
     trans_bandwidth : float
         Width of the transition band in Hz.
         Only used for ``method='fir'``.
-    %(method-fir)s
+    %(method_fir)s
         'spectrum_fit' will use multi-taper estimation of sinusoidal
         components. If freqs=None and method='spectrum_fit', significant
         sinusoidal components are detected using an F test, and noted by
@@ -1115,14 +1110,15 @@ def notch_filter(x, Fs, freqs, filter_length='auto', notch_widths=None,
     %(picks_nostr)s
         Only supported for 2D (n_channels, n_times) and 3D
         (n_epochs, n_channels, n_times) data.
-    %(n_jobs-fir)s
+    %(n_jobs_fir)s
     copy : bool
         If True, a copy of x, filtered, is returned. Otherwise, it operates
         on x in place.
     %(phase)s
     %(fir_window)s
     %(fir_design)s
-    %(pad-fir)s
+    %(pad_fir)s
+        The default is ``'reflect_limited'``.
     %(verbose)s
 
     Returns
@@ -1158,6 +1154,7 @@ def notch_filter(x, Fs, freqs, filter_length='auto', notch_widths=None,
     & Hemant Bokil, Oxford University Press, New York, 2008. Please
     cite this in publications if method 'spectrum_fit' is used.
     """
+    x = _check_filterable(x, 'notch filtered', 'notch_filter')
     iir_params, method = _check_method(method, iir_params, ['spectrum_fit'])
 
     if freqs is not None:
@@ -1205,6 +1202,7 @@ def _get_window_thresh(n_times, sfreq, mt_bandwidth, p_value):
     # but if we have a new enough scipy,
     # it's only ~0.175 sec for 8 tapers even with 100000 samples
     from scipy import stats
+    from .time_frequency.multitaper import _compute_mt_params
     dpss_n_times_max = 100000
 
     # figure out what tapers to use
@@ -1221,7 +1219,6 @@ def _mt_spectrum_proc(x, sfreq, line_freqs, notch_widths, mt_bandwidth,
                       p_value, picks, n_jobs, copy, filter_length):
     """Call _mt_spectrum_remove."""
     # set up array for filtering, reshape to 2D, operate on last axis
-    n_jobs = check_n_jobs(n_jobs)
     x, orig_shape, picks = _prep_for_filtering(x, copy, picks)
     if isinstance(filter_length, str) and filter_length == 'auto':
         filter_length = '10s'
@@ -1232,6 +1229,7 @@ def _mt_spectrum_proc(x, sfreq, line_freqs, notch_widths, mt_bandwidth,
         _get_window_thresh, sfreq=sfreq, mt_bandwidth=mt_bandwidth,
         p_value=p_value)
     window_fun, threshold = get_wt(filter_length)
+    parallel, p_fun, n_jobs = parallel_func(_mt_spectrum_remove_win, n_jobs)
     if n_jobs == 1:
         freq_list = list()
         for ii, x_ in enumerate(x):
@@ -1241,7 +1239,6 @@ def _mt_spectrum_proc(x, sfreq, line_freqs, notch_widths, mt_bandwidth,
                     get_wt)
                 freq_list.append(f)
     else:
-        parallel, p_fun, _ = parallel_func(_mt_spectrum_remove_win, n_jobs)
         data_new = parallel(p_fun(x_, sfreq, line_freqs, notch_widths,
                                   window_fun, threshold, get_wt)
                             for xi, x_ in enumerate(x)
@@ -1300,6 +1297,7 @@ def _mt_spectrum_remove(x, sfreq, line_freqs, notch_widths,
     Based on Chronux. If line_freqs is specified, all freqs within notch_width
     of each line_freq is set to zero.
     """
+    from .time_frequency.multitaper import _mt_spectra
     assert x.ndim == 1
     if x.shape[-1] != window_fun.shape[-1]:
         window_fun, threshold = get_thresh(x.shape[-1])
@@ -1370,7 +1368,27 @@ def _mt_spectrum_remove(x, sfreq, line_freqs, notch_widths,
     return x - datafit, rm_freqs
 
 
-def _check_filterable(x, kind='filtered'):
+def _check_filterable(x, kind='filtered', alternative='filter'):
+    # Let's be fairly strict about this -- users can easily coerce to ndarray
+    # at their end, and we already should do it internally any time we are
+    # using these low-level functions. At the same time, let's
+    # help people who might accidentally use low-level functions that they
+    # shouldn't use by pushing them in the right direction
+    from .io.base import BaseRaw
+    from .epochs import BaseEpochs
+    from .evoked import Evoked
+    if isinstance(x, (BaseRaw, BaseEpochs, Evoked)):
+        try:
+            name = x.__class__.__name__
+        except Exception:
+            pass
+        else:
+            raise TypeError(
+                'This low-level function only operates on np.ndarray '
+                f'instances. To get a {kind} {name} instance, use a method '
+                f'like `inst_new = inst.copy().{alternative}(...)` '
+                'instead.')
+    _validate_type(x, (np.ndarray, list, tuple), f'Data to be {kind}')
     x = np.asanyarray(x)
     if x.dtype != np.float64:
         raise ValueError('Data to be %s must be real floating, got %s'
@@ -1384,8 +1402,8 @@ def _resamp_ratio_len(up, down, n):
 
 
 @verbose
-def resample(x, up=1., down=1., npad=100, axis=-1, window='boxcar', n_jobs=1,
-             pad='reflect_limited', verbose=None):
+def resample(x, up=1., down=1., npad=100, axis=-1, window='boxcar',
+             n_jobs=None, pad='reflect_limited', *, verbose=None):
     """Resample an array.
 
     Operates along the last dimension of the array.
@@ -1401,9 +1419,9 @@ def resample(x, up=1., down=1., npad=100, axis=-1, window='boxcar', n_jobs=1,
     %(npad)s
     axis : int
         Axis along which to resample (default is the last axis).
-    %(window-resample)s
-    %(n_jobs-cuda)s
-    %(pad-fir)s
+    %(window_resample)s
+    %(n_jobs_cuda)s
+    %(pad)s
         The default is ``'reflect_limited'``.
 
         .. versionadded:: 0.15
@@ -1428,7 +1446,7 @@ def resample(x, up=1., down=1., npad=100, axis=-1, window='boxcar', n_jobs=1,
     up=up/down and down=1.
     """
     from scipy.signal import get_window
-    ifftshift, fftfreq = _import_fft(('ifftshift', 'fftfreq'))
+    from scipy.fft import ifftshift, fftfreq
     # check explicitly for backwards compatibility
     if not isinstance(axis, int):
         err = ("The axis parameter needs to be an integer (got %s). "
@@ -1438,7 +1456,7 @@ def resample(x, up=1., down=1., npad=100, axis=-1, window='boxcar', n_jobs=1,
         raise TypeError(err)
 
     # make sure our arithmetic will work
-    x = _check_filterable(x, 'resampled')
+    x = _check_filterable(x, 'resampled', 'resample')
     ratio, final_len = _resamp_ratio_len(up, down, x.shape[axis])
     del up, down
     if axis < 0:
@@ -1494,13 +1512,13 @@ def resample(x, up=1., down=1., npad=100, axis=-1, window='boxcar', n_jobs=1,
 
     # do the resampling using an adaptation of scipy's FFT-based resample()
     # use of the 'flat' window is recommended for minimal ringing
+    parallel, p_fun, n_jobs = parallel_func(_fft_resample, n_jobs)
     if n_jobs == 1:
         y = np.zeros((len(x_flat), new_len - to_removes.sum()), dtype=x.dtype)
         for xi, x_ in enumerate(x_flat):
             y[xi] = _fft_resample(x_, new_len, npads, to_removes,
                                   cuda_dict, pad)
     else:
-        parallel, p_fun, _ = parallel_func(_fft_resample, n_jobs)
         y = parallel(p_fun(x_, new_len, npads, to_removes, cuda_dict, pad)
                      for x_ in x_flat)
         y = np.array(y)
@@ -1856,7 +1874,7 @@ class FilterMixin(object):
             instead of FIR/IIR filtering. This parameter is thus used to
             determine the length of the window over which a 5th-order
             polynomial smoothing is used.
-        %(verbose_meth)s
+        %(verbose)s
 
         Returns
         -------
@@ -1904,10 +1922,10 @@ class FilterMixin(object):
 
     @verbose
     def filter(self, l_freq, h_freq, picks=None, filter_length='auto',
-               l_trans_bandwidth='auto', h_trans_bandwidth='auto', n_jobs=1,
+               l_trans_bandwidth='auto', h_trans_bandwidth='auto', n_jobs=None,
                method='fir', iir_params=None, phase='zero',
                fir_window='hamming', fir_design='firwin',
-               skip_by_annotation=('edge', 'bad_acq_skip'), pad='edge',
+               skip_by_annotation=('edge', 'bad_acq_skip'), pad='edge', *,
                verbose=None):
         """Filter a subset of channels.
 
@@ -1919,8 +1937,8 @@ class FilterMixin(object):
         %(filter_length)s
         %(l_trans_bandwidth)s
         %(h_trans_bandwidth)s
-        %(n_jobs-fir)s
-        %(method-fir)s
+        %(n_jobs_fir)s
+        %(method_fir)s
         %(iir_params)s
         %(phase)s
         %(fir_window)s
@@ -1936,8 +1954,8 @@ class FilterMixin(object):
             To disable, provide an empty list. Only used if ``inst`` is raw.
 
             .. versionadded:: 0.16.
-        %(pad-fir)s
-        %(verbose_meth)s
+        %(pad_fir)s
+        %(verbose)s
 
         Returns
         -------
@@ -1976,7 +1994,7 @@ class FilterMixin(object):
 
         .. note:: If n_jobs > 1, more memory is required as
                   ``len(picks) * n_times`` additional time points need to
-                  be temporaily stored in memory.
+                  be temporarily stored in memory.
 
         For more information, see the tutorials
         :ref:`disc-filtering` and :ref:`tut-filter-resample` and
@@ -2015,8 +2033,8 @@ class FilterMixin(object):
         return self
 
     @verbose
-    def resample(self, sfreq, npad='auto', window='boxcar', n_jobs=1,
-                 pad='edge', verbose=None):  # lgtm
+    def resample(self, sfreq, npad='auto', window='boxcar', n_jobs=None,
+                 pad='edge', *, verbose=None):
         """Resample data.
 
         If appropriate, an anti-aliasing filter is applied before resampling.
@@ -2029,14 +2047,14 @@ class FilterMixin(object):
         sfreq : float
             New sample rate to use.
         %(npad)s
-        %(window-resample)s
-        %(n_jobs-cuda)s
-        %(pad-fir)s
+        %(window_resample)s
+        %(n_jobs_cuda)s
+        %(pad)s
             The default is ``'edge'``, which pads with the edge values of each
             vector.
 
             .. versionadded:: 0.15
-        %(verbose_meth)s
+        %(verbose)s
 
         Returns
         -------
@@ -2072,17 +2090,14 @@ class FilterMixin(object):
         new_times = (np.arange(self._data.shape[-1], dtype=np.float64) /
                      sfreq + self.times[0])
         # adjust indirectly affected variables
-        if isinstance(self, BaseEpochs):
-            self._set_times(new_times)
-            self._raw_times = self.times
-        else:  # isinstance(self, Evoked)
-            self.times = new_times
-            self._update_first_last()
+        self._set_times(new_times)
+        self._raw_times = self.times
+        self._update_first_last()
         return self
 
     @verbose
-    def apply_hilbert(self, picks=None, envelope=False, n_jobs=1, n_fft='auto',
-                      verbose=None):
+    def apply_hilbert(self, picks=None, envelope=False, n_jobs=None,
+                      n_fft='auto', *, verbose=None):
         """Compute analytic signal or envelope for a subset of channels.
 
         Parameters
@@ -2097,7 +2112,7 @@ class FilterMixin(object):
             will be padded with zeros before computing Hilbert, then cut back
             to original length. If None, n == self.n_times. If 'auto',
             the next highest fast FFT length will be use.
-        %(verbose_meth)s
+        %(verbose)s
 
         Returns
         -------
@@ -2125,7 +2140,7 @@ class FilterMixin(object):
         If envelope=False, more memory is required since the original raw data
         as well as the analytic signal have temporarily to be stored in memory.
         If n_jobs > 1, more memory is required as ``len(picks) * n_times``
-        additional time points need to be temporaily stored in memory.
+        additional time points need to be temporarily stored in memory.
 
         Also note that the ``n_fft`` parameter will allow you to pad the signal
         with zeros before performing the Hilbert transform. This padding
@@ -2166,6 +2181,7 @@ class FilterMixin(object):
         if dtype is not None and dtype != self._data.dtype:
             self._data = self._data.astype(dtype)
 
+        parallel, p_fun, n_jobs = parallel_func(_check_fun, n_jobs)
         if n_jobs == 1:
             # modify data inplace to save memory
             for idx in picks:
@@ -2173,7 +2189,6 @@ class FilterMixin(object):
                     _my_hilbert, data_in[..., idx, :], *args, **kwargs)
         else:
             # use parallel function
-            parallel, p_fun, _ = parallel_func(_check_fun, n_jobs)
             data_picks_new = parallel(
                 p_fun(_my_hilbert, data_in[..., p, :], *args, **kwargs)
                 for p in picks)
@@ -2257,7 +2272,7 @@ def design_mne_c_filter(sfreq, l_freq=None, h_freq=40.,
     4197 frequencies are directly constructed, with zeroes in the stop-band
     and ones in the passband, with squared cosine ramps in between.
     """
-    irfft = _import_fft('irfft')
+    from scipy.fft import irfft
     n_freqs = (4096 + 2 * 2048) // 2 + 1
     freq_resp = np.ones(n_freqs)
     l_freq = 0 if l_freq is None else float(l_freq)
