@@ -9,10 +9,11 @@ import os.path as op
 import numpy as np
 
 from ._eeglab import _readmat
-from ..pick import _PICK_TYPES_KEYS
-from ..utils import _read_segments_file, _find_channels
+from .._digitization import _ensure_fiducials_head
 from ..constants import FIFF
 from ..meas_info import create_info
+from ..pick import _PICK_TYPES_KEYS
+from ..utils import _read_segments_file, _find_channels
 from ..base import BaseRaw
 from ...utils import logger, verbose, warn, fill_doc, Bunch, _check_fname
 from ...channels import make_dig_montage
@@ -159,6 +160,7 @@ def _get_montage_information(eeg, get_pos):
         montage = make_dig_montage(
             ch_pos=dict(zip(ch_names, np.array(pos))),
             coord_frame='head', lpa=lpa, rpa=rpa, nasion=nasion)
+        _ensure_fiducials_head(montage.dig)
     else:
         montage = None
 
