@@ -19,23 +19,24 @@ from numpy.testing import assert_allclose, assert_array_equal
 from mne import (read_source_estimate, read_evokeds, read_cov,
                  read_forward_solution, pick_types_forward,
                  SourceEstimate, MixedSourceEstimate, write_surface,
-                 VolSourceEstimate, vertex_to_mni)
+                 VolSourceEstimate, vertex_to_mni, Dipole)
 from mne.minimum_norm import apply_inverse, make_inverse_operator
 from mne.source_space import (read_source_spaces,
                               setup_volume_source_space)
 from mne.datasets import testing
 from mne.fixes import _cell_data
 from mne.io import read_info
-from mne.utils import check_version, requires_pysurfer
+from mne.utils import check_version
 from mne.label import read_label
 from mne.viz._brain import Brain, _LinkViewer, _BrainScraper, _LayeredMesh
 from mne.viz._brain.colormap import calculate_lut
+from mne.viz.utils import _get_cmap
 
-from matplotlib import cm, image
+from matplotlib import image
 from matplotlib.lines import Line2D
 
 data_path = testing.data_path(download=False)
-subject_id = 'sample'
+subject = 'sample'
 subjects_dir = op.join(data_path, 'subjects')
 sample_dir = op.join(data_path, 'MEG', 'sample')
 fname_raw_testing = op.join(sample_dir, 'sample_audvis_trunc_raw.fif')
@@ -45,7 +46,7 @@ fname_label = op.join(sample_dir, 'labels', 'Vis-lh.label')
 fname_cov = op.join(sample_dir, 'sample_audvis_trunc-cov.fif')
 fname_evoked = op.join(sample_dir, 'sample_audvis_trunc-ave.fif')
 fname_fwd = op.join(sample_dir, 'sample_audvis_trunc-meg-eeg-oct-4-fwd.fif')
-src_fname = op.join(subjects_dir, subject_id, 'bem', 'sample-oct-6-src.fif')
+src_fname = op.join(subjects_dir, subject, 'bem', 'sample-oct-6-src.fif')
 
 
 class _Collection(object):
@@ -156,15 +157,11 @@ def test_brain_gc(renderer_pyvistaqt, brain_gc):
     brain.close()
 
 
-@requires_pysurfer
 @testing.requires_testing_data
 def test_brain_routines(renderer, brain_gc):
     """Test backend agnostic Brain routines."""
     brain_klass = renderer.get_brain_class()
-    if renderer.get_3d_backend() == "mayavi":
-        from surfer import Brain
-    else:  # PyVista
-        from mne.viz._brain import Brain
+    from mne.viz._brain import Brain
     assert brain_klass == Brain
 
 
@@ -182,7 +179,7 @@ def test_brain_init(renderer_pyvistaqt, tmp_path, pixel_ratio, brain_gc):
     title = 'test'
     size = (300, 300)
 
-    kwargs = dict(subject_id=subject_id, subjects_dir=subjects_dir)
+    kwargs = dict(subject=subject, subjects_dir=subjects_dir)
     with pytest.raises(ValueError, match='"size" parameter must be'):
         Brain(hemi=hemi, surf=surf, size=[1, 2, 3], **kwargs)
     with pytest.raises(ValueError, match='.*hemi.*Allowed values.*'):
@@ -196,12 +193,16 @@ def test_brain_init(renderer_pyvistaqt, tmp_path, pixel_ratio, brain_gc):
     with pytest.raises(ValueError, match='interaction'):
         Brain(hemi=hemi, surf=surf, interaction='foo', **kwargs)
     with pytest.raises(FileNotFoundError, match=r'lh\.whatever'):
-        Brain(subject_id, 'lh', 'whatever')
+        Brain(hemi='lh', surf='whatever', **kwargs)
     with pytest.raises(ValueError, match='`surf` cannot be seghead'):
         Brain(hemi='lh', surf='seghead', **kwargs)
     with pytest.raises(ValueError, match='RGB argument'):
         Brain('sample', cortex='badcolor')
-    Brain(subject_id, hemi=None, surf=None)  # test no surfaces
+    # test no surfaces
+    with pytest.warns(FutureWarning, match='show_toolbar'):  # and subject_id
+        Brain(subject_id=subject, hemi=None, surf=None, show_toolbar=True)
+    with pytest.raises(TypeError, match='missing 1 required positional'):
+        Brain()
     renderer_pyvistaqt.backend._close_all()
 
     brain = Brain(hemi=hemi, surf=surf, size=size, title=title,
@@ -347,6 +348,30 @@ def test_brain_init(renderer_pyvistaqt, tmp_path, pixel_ratio, brain_gc):
     with pytest.raises(RuntimeError, match='must be "meg", "head" or "mri"'):
         brain.add_sensors(info, trans=fname_trans)
 
+    # add dipole
+    dip = Dipole(times=[0], pos=[[-0.06439933, 0.00733009, 0.06280205]],
+                 amplitude=[3e-8], ori=[[0, 1, 0]], gof=50)
+    brain.add_dipole(dip, fname_trans, colors='blue', scales=5, alpha=0.5)
+    brain.remove_dipole()
+
+    with pytest.raises(ValueError, match='The number of colors'):
+        brain.add_dipole(dip, fname_trans, colors=['red', 'blue'])
+
+    with pytest.raises(ValueError, match='The number of scales'):
+        brain.add_dipole(dip, fname_trans, scales=[1, 2])
+
+    fwd = read_forward_solution(fname_fwd)
+    brain.add_forward(fwd, fname_trans, alpha=0.5, scale=10)
+    brain.remove_forward()
+
+    # fake incorrect coordinate frame
+    fwd['coord_frame'] = 99
+    with pytest.raises(RuntimeError, match='must be "head" or "mri"'):
+        brain.add_forward(fwd, fname_trans)
+    fwd['coord_frame'] = 2003
+    with pytest.raises(RuntimeError, match='must be "head" or "mri"'):
+        brain.add_forward(fwd, fname_trans)
+
     # add text
     brain.add_text(x=0, y=0, text='foo')
     with pytest.raises(ValueError, match='already exists'):
@@ -363,14 +388,14 @@ def test_brain_init(renderer_pyvistaqt, tmp_path, pixel_ratio, brain_gc):
     borders = [True, 2]
     alphas = [1, 0.5]
     colors = [None, 'r']
-    brain = Brain(subject_id='fsaverage', hemi='both', size=size,
+    brain = Brain(subject='fsaverage', hemi='both', size=size,
                   surf='inflated', subjects_dir=subjects_dir)
     with pytest.raises(RuntimeError, match="both hemispheres"):
         brain.add_annotation(annots[-1])
     with pytest.raises(ValueError, match="does not exist"):
         brain.add_annotation('foo')
     brain.close()
-    brain = Brain(subject_id='fsaverage', hemi=hemi, size=size,
+    brain = Brain(subject='fsaverage', hemi=hemi, size=size,
                   surf='inflated', subjects_dir=subjects_dir)
     for a, b, p, color in zip(annots, borders, alphas, colors):
         brain.add_annotation(a, b, p, color=color)
@@ -382,10 +407,18 @@ def test_brain_init(renderer_pyvistaqt, tmp_path, pixel_ratio, brain_gc):
     assert_allclose(cam.GetFocalPoint(), view_args["focalpoint"])
     assert_allclose(cam.GetDistance(), view_args["distance"])
     assert_allclose(cam.GetRoll(), previous_roll + view_args["roll"])
-    del view_args
 
-    with pytest.warns(DeprecationWarning, match='`view` is a dict'):
-        brain.show_view(view=dict(azimuth=180., elevation=90.))
+    # test get_view
+    azimuth, elevation = 180., 90.
+    view_args.update(azimuth=azimuth, elevation=elevation)
+    brain.show_view(**view_args)
+    roll, distance, azimuth, elevation, focalpoint = brain.get_view()
+    assert_allclose(cam.GetRoll(), roll)
+    assert_allclose(cam.GetDistance(), distance)
+    assert_allclose(view_args['azimuth'] % 360, azimuth % 360)
+    assert_allclose(view_args['elevation'] % 180, elevation % 180)
+    assert_allclose(view_args['focalpoint'], focalpoint)
+    del view_args
 
     # image and screenshot
     fname = op.join(str(tmp_path), 'test.png')
@@ -395,18 +428,21 @@ def test_brain_init(renderer_pyvistaqt, tmp_path, pixel_ratio, brain_gc):
     fp = np.array(
         brain._renderer.figure.plotter.renderer.ComputeVisiblePropBounds())
     fp = (fp[1::2] + fp[::2]) * 0.5
-    azimuth, elevation = 180., 90.
     for view_args in (dict(azimuth=azimuth, elevation=elevation,
                            focalpoint='auto'),
                       dict(view='lateral', hemi='lh')):
         brain.show_view(**view_args)
-        assert_allclose(brain._renderer.figure._azimuth, azimuth)
-        assert_allclose(brain._renderer.figure._elevation, elevation)
+        assert_allclose(brain._renderer.figure._azimuth % 360, azimuth % 360)
+        assert_allclose(
+            brain._renderer.figure._elevation % 180, elevation % 180)
         assert_allclose(cam.GetFocalPoint(), fp)
     del view_args
     img = brain.screenshot(mode='rgba')
     want_size = np.array([size[0] * pixel_ratio, size[1] * pixel_ratio, 4])
-    assert_allclose(img.shape, want_size)
+    # on macOS sometimes matplotlib is HiDPI and VTK is not...
+    div = 2 if np.allclose(img.shape[:2], want_size[:2] / 2., atol=15) else 1
+    want_size[:2] /= div
+    assert_allclose(img.shape, want_size, atol=15)
     brain.close()
 
 
@@ -437,8 +473,6 @@ def test_single_hemi(hemi, renderer_interactive_pyvistaqt, brain_gc):
 @pytest.mark.slowtest
 def test_brain_save_movie(tmp_path, renderer, brain_gc):
     """Test saving a movie of a Brain instance."""
-    if renderer._get_3d_backend() == "mayavi":
-        pytest.skip('Save movie only supported on PyVista')
     from imageio_ffmpeg import count_frames_and_secs
     brain = _create_testing_brain(hemi='lh', time_viewer=False,
                                   cortex=['r', 'b'])  # custom binarized
@@ -609,7 +643,7 @@ def test_brain_time_viewer(renderer_interactive_pyvistaqt, pixel_ratio,
     brain.show_view(azimuth=180., elevation=90.)
     img = brain.screenshot(mode='rgb')
     want_shape = np.array([300 * pixel_ratio, 300 * pixel_ratio, 3])
-    assert_allclose(img.shape, want_shape)
+    assert_allclose(img.shape, want_shape, atol=30)
     brain.close()
 
 
@@ -709,8 +743,10 @@ def test_brain_traces(renderer_interactive_pyvistaqt, hemi, src, tmp_path,
     assert hasattr(brain, "_spheres")
     assert brain._scalar_bar.GetNumberOfLabels() == 3
 
-    # add foci should work for volumes
-    brain.add_foci([[0, 0, 0]], hemi='lh' if src == 'surface' else 'vol')
+    # add foci should work for 'lh', 'rh' and 'vol'
+    for current_hemi in hemi_str:
+        brain.add_foci([[0, 0, 0]], hemi=current_hemi)
+        assert_array_equal(brain._data[current_hemi]['foci'], [[0, 0, 0]])
 
     # test points picked by default
     picked_points = brain.get_picked_points()
@@ -774,7 +810,7 @@ def test_brain_traces(renderer_interactive_pyvistaqt, hemi, src, tmp_path,
         mni = vertex_to_mni(
             vertices=vertex_id,
             hemis=hemi_int,
-            subject=brain._subject_id,
+            subject=brain._subject,
             subjects_dir=brain._subjects_dir
         )
         label = "{}:{} MNI: {}".format(
@@ -821,6 +857,41 @@ something
         assert img.shape[:2] == screenshot_all.shape[:2]
 
 
+# TODO: don't skip on Windows, see
+# https://github.com/mne-tools/mne-python/pull/10935
+# for some reason there is a dependency issue with ipympl even using pyvista
+@pytest.mark.skipif(sys.platform == 'win32', reason='ipympl issue on Windows')
+@testing.requires_testing_data
+def test_brain_scraper(renderer_interactive_pyvistaqt, brain_gc, tmp_path):
+    """Test a simple scraping example."""
+    pytest.importorskip('sphinx_gallery')
+    stc = read_source_estimate(fname_stc, subject='sample')
+    size = (600, 300)
+    brain = stc.plot(subjects_dir=subjects_dir,
+                     time_viewer=True, show_traces=True,
+                     hemi='split', size=size, views='lat')
+    fnames = [str(tmp_path / f'temp_{ii}.png') for ii in range(2)]
+    block_vars = dict(image_path_iterator=iter(fnames),
+                      example_globals=dict(brain=brain))
+    block = ('code', '', 1)
+    gallery_conf = dict(src_dir=str(tmp_path), compress_images=[])
+    scraper = _BrainScraper()
+    rst = scraper(block, block_vars, gallery_conf)
+    assert brain.plotter is None  # closed
+    assert brain._cleaned
+    del brain
+    fname = fnames[0]
+    assert op.basename(fname) in rst
+    assert op.isfile(fname)
+    img = image.imread(fname)
+    w = img.shape[1]
+    w0 = size[0]
+    # With matplotlib 3.6 on Linux+conda we get a width of 624,
+    # similar tweak in test_brain_init above
+    assert np.isclose(w, w0, atol=30) or \
+        np.isclose(w, w0 * 2, atol=30), f'w ∉ {{{w0}, {2 * w0}}}'  # HiDPI
+
+
 @testing.requires_testing_data
 @pytest.mark.slowtest
 def test_brain_linkviewer(renderer_interactive_pyvistaqt, brain_gc):
@@ -868,11 +939,10 @@ def test_calculate_lut():
     calculate_lut(colormap, alpha=alpha, fmin=fmin,
                   fmid=fmid, fmax=fmax, center=center)
     center = 0.0
-    colormap = cm.get_cmap(colormap)
-    calculate_lut(colormap, alpha=alpha, fmin=fmin,
+    cmap = _get_cmap(colormap)
+    calculate_lut(cmap, alpha=alpha, fmin=fmin,
                   fmid=fmid, fmax=fmax, center=center)
 
-    cmap = cm.get_cmap(colormap)
     zero_alpha = np.array([1., 1., 1., 0])
     half_alpha = np.array([1., 1., 1., 0.5])
     atol = 1.5 / 256.
@@ -978,12 +1048,12 @@ def _create_testing_brain(hemi, surf='inflated', src='surface',
             evoked.info, fwd, noise_cov, loose=1.)
         stc = apply_inverse(evoked, free, pick_ori='vector')
         return stc.plot(
-            subject=subject_id, hemi=hemi, size=size,
+            subject=subject, hemi=hemi, size=size,
             subjects_dir=subjects_dir, colormap='auto',
             **kwargs)
     if src in ('volume', 'mixed'):
         vol_src = setup_volume_source_space(
-            subject_id, 7., mri='aseg.mgz',
+            subject, 7., mri='aseg.mgz',
             volume_label='Left-Cerebellum-Cortex',
             subjects_dir=subjects_dir, add_interpolator=False)
         assert len(vol_src) == 1
@@ -1014,8 +1084,17 @@ def _create_testing_brain(hemi, surf='inflated', src='surface',
         clim['pos_lims'] = clim.pop('lims')
 
     brain_data = getattr(stc, meth)(
-        subject=subject_id, hemi=hemi, surface=surf, size=size,
+        subject=subject, hemi=hemi, surface=surf, size=size,
         subjects_dir=subjects_dir, colormap='auto',
         clim=clim, src=sample_src,
         **kwargs)
     return brain_data
+
+
+def test_foci_mapping(tmp_path, renderer_interactive_pyvistaqt):
+    """Test mapping foci to the surface."""
+    tiny_brain, _ = tiny(tmp_path)
+    foci_coords = tiny_brain.geo['lh'].coords[:2] + 0.01
+    tiny_brain.add_foci(foci_coords, map_surface='white')
+    assert_array_equal(tiny_brain._data['lh']['foci'],
+                       tiny_brain.geo['lh'].coords[:2])
