@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 """
+.. _tut-ecd-dipole:
+
 ============================================================
 Source localization with equivalent current dipole (ECD) fit
 ============================================================
@@ -25,13 +27,12 @@ from nilearn.plotting import plot_anat
 from nilearn.datasets import load_mni152_template
 
 data_path = mne.datasets.sample.data_path()
-subjects_dir = op.join(data_path, 'subjects')
-fname_ave = op.join(data_path, 'MEG', 'sample', 'sample_audvis-ave.fif')
-fname_cov = op.join(data_path, 'MEG', 'sample', 'sample_audvis-cov.fif')
-fname_bem = op.join(subjects_dir, 'sample', 'bem', 'sample-5120-bem-sol.fif')
-fname_trans = op.join(data_path, 'MEG', 'sample',
-                      'sample_audvis_raw-trans.fif')
-fname_surf_lh = op.join(subjects_dir, 'sample', 'surf', 'lh.white')
+subjects_dir = data_path / 'subjects'
+fname_ave = data_path / 'MEG' / 'sample' / 'sample_audvis-ave.fif'
+fname_cov = data_path / 'MEG' / 'sample' / 'sample_audvis-cov.fif'
+fname_bem = subjects_dir / 'sample' / 'bem' / 'sample-5120-bem-sol.fif'
+fname_trans = data_path / 'MEG' / 'sample' / 'sample_audvis_raw-trans.fif'
+fname_surf_lh = subjects_dir / 'sample' / 'surf' / 'lh.white'
 
 # %%
 # Let's localize the N100m (using MEG only)
@@ -47,6 +48,16 @@ dip = mne.fit_dipole(evoked, fname_cov, fname_bem, fname_trans)[0]
 # Plot the result in 3D brain with the MRI image.
 dip.plot_locations(fname_trans, 'sample', subjects_dir, mode='orthoview')
 
+# %%
+# We can also plot the result using outlines of the head and brain.
+
+# sphinx_gallery_thumbnail_number = 2
+
+color = ['k'] * len(dip)
+color[np.argmax(dip.gof)] = 'r'
+dip.plot_locations(fname_trans, 'sample', subjects_dir, mode='outlines',
+                   color=color)
+
 # %%
 # Plot the result in 3D brain with the MRI image using Nilearn
 # In MRI coordinates and in MNI coordinates (template brain)
@@ -95,8 +106,8 @@ fig, axes = plt.subplots(nrows=1, ncols=4, figsize=[10., 3.4],
 vmin, vmax = -400, 400  # make sure each plot has same colour range
 
 # first plot the topography at the time of the best fitting (single) dipole
-plot_params = dict(times=best_time, ch_type='mag', outlines='skirt',
-                   colorbar=False, time_unit='s')
+plot_params = dict(times=best_time, ch_type='mag', outlines='head',
+                   colorbar=False)
 evoked.plot_topomap(time_format='Measured field', axes=axes[0], **plot_params)
 
 # compare this to the predicted field
@@ -116,7 +127,7 @@ fig.tight_layout()
 # orientation (the one that maximized GOF) over the entire interval
 dip_fixed = mne.fit_dipole(evoked_full, fname_cov, fname_bem, fname_trans,
                            pos=dip.pos[best_idx], ori=dip.ori[best_idx])[0]
-dip_fixed.plot(time_unit='s')
+dip_fixed.plot()
 
 # %%
 # References
